<?php

namespace Drupal\rest_role_restrict\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Restricts REST API access based on user roles.
 */
class RestRoleAccessSubscriber implements EventSubscriberInterface
{

    /**
     * The current user.
     *
     * @var \Drupal\Core\Session\AccountProxyInterface
     */
    protected $currentUser;

    /**
     * The config factory.
     *
     * @var \Drupal\Core\Config\ConfigFactoryInterface
     */
    protected $configFactory;

    /**
     * The request stack.
     *
     * @var \Symfony\Component\HttpFoundation\RequestStack
     */
    protected $requestStack;

    /**
     * Constructs a new RestRoleAccessSubscriber object.
     *
     * @param \Drupal\Core\Session\AccountProxyInterface $current_user
     *   The current user.
     * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
     *   The config factory.
     * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
     *   The request stack.
     */
    public function __construct(AccountProxyInterface $current_user, ConfigFactoryInterface $config_factory, RequestStack $request_stack)
    {
        $this->currentUser = $current_user;
        $this->configFactory = $config_factory;
        $this->requestStack = $request_stack;
    }

    /**
     * Restrict access to the REST API based on roles.
     *
     * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
     *   The event to process.
     */
    public function onKernelRequest(RequestEvent $event)
    {
        $request = $event->getRequest();

        // Only restrict requests that are hitting REST routes.
        if (strpos($request->getPathInfo(), '/api/') === false && !$request->attributes->get('_rest_resource_config')) {
            return;
        }

        $user = $this->currentUser;
        $roles = $user->getRoles();

        $config = $this->configFactory->get('rest_role_restrict.settings');
        $allowed_roles_raw = $config->get('allowed_roles') ?? [];

        // Normalize config in case it’s structured like: allowed_roles: { restapi: restapi }
        $allowed_roles = array_keys($allowed_roles_raw);

        // Check if user has at least one allowed role.
        $has_access = (bool)array_intersect($roles, $allowed_roles);

        if (!$has_access) {
            \Drupal::logger('rest_role_restrict')->notice(
                'API Access Denied => User ID: @uid | Roles: @roles',
                [
                    '@uid' => $user->id(),
                    '@roles' => implode(', ', $roles),
                ]
            );
            throw new AccessDeniedHttpException('Access to the REST API is restricted to specific roles. Contact your site administrator.');
        }
    }

    /**
     * {@inheritdoc}
     */
    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::REQUEST => ['onKernelRequest', 28],
        ];
    }

}

